==> react-static-plugin-typescript-2.1.0.tgz: package/src/node.api.ts <==
import { Configuration as WebpackConfiguration, RuleSetRule, RuleSetLoader } from 'webpack';
import ForkTsCheckerWebpackPlugin from 'fork-ts-checker-webpack-plugin';
import { resolve, resolveSync } from 'tsconfig';

interface Options {
  typeCheck?: boolean;
  tsconfig?: string;
};

export default (options?: Options) => {
  return {
    config: getReactStaticConfigExtender(),
    webpack: getWebpackConfigExtender(options),
  };
};

interface ReactStaticArgs {
  stage: 'prod' | 'dev' | 'node';
  defaultLoaders: {
    jsLoader: RuleSetRule & { use: Array<RuleSetLoader & { options: object }>};
    jsLoaderExt: RuleSetRule;
    cssLoader: RuleSetRule;
    fileLoader: RuleSetRule;
  },
};

function getWebpackConfigExtender(options?: Options) {
  // The `|| undefined` is there to compensate for `resolve` possibly returning
  // `void`, which TypeScript complains about not being `undefined`...
  const tsconfigPath = resolveSync('./', (options || {}).tsconfig) || /* istanbul ignore next */ undefined;

  return (previousConfig: WebpackConfiguration, args: ReactStaticArgs) => {
    const previousModuleConfig = previousConfig.module || {};
    const previousResolveConfig = previousConfig.resolve || {};
    const previousResolveExtensions = previousResolveConfig.extensions || [];

    const typescriptLoader = getTypeScriptLoader(args.defaultLoaders.jsLoader);

    const plugins = previousConfig.plugins || [];
    if ((options || {}).typeCheck === true) {
      plugins.push(getTypecheckPlugin(tsconfigPath));
    }

    return {
      ...previousConfig,
      plugins: plugins,
      resolve: {
        ...previousResolveConfig,
        extensions: previousResolveExtensions.concat(['.ts', '.tsx']),
      },
      module: {
        ...previousModuleConfig,
        rules: [{
          oneOf: [
            typescriptLoader,
            args.defaultLoaders.jsLoaderExt,
            args.defaultLoaders.cssLoader,
            args.defaultLoaders.fileLoader,
          ],
        }],
      },
    };
  };
}

type ReactStaticConfig = {
  extensions: string[];
};
function getReactStaticConfigExtender() {
  return (previousConfig: ReactStaticConfig) => {

    return {
      ...previousConfig,
      extensions: previousConfig.extensions.concat(['.ts', '.tsx']),
    };
  };
}

function getTypeScriptLoader(jsLoader: RuleSetRule & { use: Array<RuleSetLoader & { options: object }>}) {
  const jsLoaderBabelPresets = jsLoader.use[0].options.presets || [];

  const typescriptLoader: RuleSetRule = {
    ...jsLoader,
    test: /\.(js|jsx|ts|tsx)$/,
    use: [
      {
        ...jsLoader.use[0],
        options: {
          ...jsLoader.use[0].options,
          presets: jsLoaderBabelPresets.concat('@babel/preset-typescript'),
        },
      },
    ],
  };

  return typescriptLoader;
}

function getTypecheckPlugin(tsconfigPath?: string) {
  return new ForkTsCheckerWebpackPlugin({
    async: false,
    checkSyntacticErrors: true,
    tsconfig: tsconfigPath,
  });
}

==> react-static-plugin-typescript-3.0.0.tgz: package/src/node.api.ts <==
import { Configuration as WebpackConfiguration, RuleSetRule, RuleSetLoader } from 'webpack';
import ForkTsCheckerWebpackPlugin from 'fork-ts-checker-webpack-plugin';
import { resolve, resolveSync } from 'tsconfig';

interface Options {
  typeCheck?: boolean;
  tsconfig?: string;
};

export default (options?: Options) => {
  return {
    config: getReactStaticConfigExtender(),
    webpack: getWebpackConfigExtender(options),
  };
};

interface ReactStaticArgs {
  stage: 'prod' | 'dev' | 'node';
  defaultLoaders: {
    jsLoader: RuleSetRule & { use: Array<RuleSetLoader & { options: object }>};
    jsLoaderExt: RuleSetRule;
    cssLoader: RuleSetRule;
    fileLoader: RuleSetRule;
  },
};

function getWebpackConfigExtender(options?: Options) {
  // The `|| undefined` is there to compensate for `resolve` possibly returning
  // `void`, which TypeScript complains about not being `undefined`...
  const tsconfigPath = resolveSync('./', (options || {}).tsconfig) || /* istanbul ignore next */ undefined;

  return (previousConfig: WebpackConfiguration, args: ReactStaticArgs) => {
    const previousModuleConfig = previousConfig.module || {};
    const previousResolveConfig = previousConfig.resolve || {};
    const previousResolveExtensions = previousResolveConfig.extensions || [];

    const typescriptLoader = getTypeScriptLoader(args.defaultLoaders.jsLoader);

    const plugins = previousConfig.plugins || [];
    if ((options || {}).typeCheck !== false) {
      plugins.push(getTypecheckPlugin(tsconfigPath));
    }

    return {
      ...previousConfig,
      plugins: plugins,
      resolve: {
        ...previousResolveConfig,
        extensions: previousResolveExtensions.concat(['.ts', '.tsx']),
      },
      module: {
        ...previousModuleConfig,
        rules: [{
          oneOf: [
            typescriptLoader,
            args.defaultLoaders.jsLoaderExt,
            args.defaultLoaders.cssLoader,
            args.defaultLoaders.fileLoader,
          ],
        }],
      },
    };
  };
}

type ReactStaticConfig = {
  extensions: string[];
};
function getReactStaticConfigExtender() {
  return (previousConfig: ReactStaticConfig) => {

    return {
      ...previousConfig,
      extensions: previousConfig.extensions.concat(['.ts', '.tsx']),
    };
  };
}

function getTypeScriptLoader(jsLoader: RuleSetRule & { use: Array<RuleSetLoader & { options: object }>}) {
  const jsLoaderBabelPresets = jsLoader.use[0].options.presets || [];

  const typescriptLoader: RuleSetRule = {
    ...jsLoader,
    test: /\.(js|jsx|ts|tsx)$/,
    use: [
      {
        ...jsLoader.use[0],
        options: {
          ...jsLoader.use[0].options,
          presets: jsLoaderBabelPresets.concat('@babel/preset-typescript'),
        },
      },
    ],
  };

  return typescriptLoader;
}

function getTypecheckPlugin(tsconfigPath?: string) {
  return new ForkTsCheckerWebpackPlugin({
    async: false,
    checkSyntacticErrors: true,
    tsconfig: tsconfigPath,
  });
}
